
#include <OgreException.h>

#include "ClientGameManager.h"
#include "ClientGameState.h"
#include "GuiManager.h"
#include "OgreConsole.h"

using namespace Ogre;

// ============================================================================
ClientGameManager::ClientGameManager() :
    mRoot( 0 ),
    bShutdown( false )
{
	// nuthin'
}
// ============================================================================
ClientGameManager::~ClientGameManager()
{
	//destroyRoot();
}
// ============================================================================
void ClientGameManager::destroyRoot()
{
	if( mRoot )
	{
        delete mRoot;
        mRoot = 0;
    }
}
// ============================================================================
void ClientGameManager::startGame( ClientGameState *gameState)
{
	srand (time(0));
  	createRoot();
	defineResources();
	setupRenderSystem();
	createRenderWindow();

	initializeResourceGroups();
	setupInputSystem();

	new OgreConsole;

	this->changeState( gameState );

	 static unsigned long lTimeLastFrame = 0;

	 while( !bShutdown )
	 {
        // Calculate time since last frame and remember current time for next frame
        unsigned long lTimeCurrentFrame = mRoot->getTimer()->getMilliseconds();
        unsigned long lTimeSinceLastFrame = lTimeCurrentFrame - lTimeLastFrame;
        lTimeLastFrame = lTimeCurrentFrame;

        // Update inputmanager
        InputManager::getSingleton().capture();

        // Update current state
        mStates.back()->update( lTimeSinceLastFrame );

		WindowEventUtilities::messagePump();//yys
		// Render next frame
        mRoot->renderOneFrame();
	 }

	 this->processShutdown();

	 // destroy!
}
// ============================================================================
void ClientGameManager::changeState( ClientGameState *gameState )
{
    // Cleanup the current state
    if( !mStates.empty() ) {
        mStates.back()->exit();
        mStates.pop_back();
    }

    // Store and init the new state
    mStates.push_back( gameState );
    mStates.back()->enter();
}
// ============================================================================
void ClientGameManager::pushState( ClientGameState *gameState )
{
    // Pause current state
    if( !mStates.empty() ) {
        mStates.back()->pause();
    }

    // Store and init the new state
    mStates.push_back( gameState );
    mStates.back()->enter();
}
// ============================================================================
void ClientGameManager::popState()
{
    // Cleanup the current state
    if( !mStates.empty() ) {
        mStates.back()->exit();
        mStates.pop_back();
    }

    // Resume previous state
    if( !mStates.empty() ) {
        mStates.back()->resume();
    }
}
// ============================================================================
void ClientGameManager::requestShutdown()
{
    bShutdown = true;
}
// ============================================================================
bool ClientGameManager::keyPressed( const OIS::KeyEvent &e )
{
    // Call keyPressed of current state
    mStates.back()->keyPressed( e );

    return true;
}
// ============================================================================
bool ClientGameManager::keyReleased( const OIS::KeyEvent &e )
{
    // Call keyReleased of current state
    mStates.back()->keyReleased( e );

    return true;
}
// ============================================================================
bool ClientGameManager::mouseMoved( const OIS::MouseEvent &e )
{
    // Call mouseMoved of current state
    mStates.back()->mouseMoved( e );

    return true;
}
// ============================================================================
bool ClientGameManager::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
    // Call mousePressed of current state
    mStates.back()->mousePressed( e, id );

    return true;
}
// ============================================================================
bool ClientGameManager::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
    // Call mouseReleased of current state
    mStates.back()->mouseReleased( e, id );

    return true;
}
// ============================================================================
ClientGameManager & ClientGameManager::getSingleton()
{
	static ClientGameManager instance;
    return instance;
}
// ============================================================================
void ClientGameManager::createRoot()
{
	//mRoot = new Root();
	mRoot = new Root("plugins.cfg", "ogre.cfg", ""); // test to disable writing to cfg files so can run on CD.
}
// ============================================================================
void ClientGameManager::defineResources()
{
	String secName, typeName, archName;
	ConfigFile cf;
	cf.load("resources.cfg");

	ConfigFile::SectionIterator seci = cf.getSectionIterator();
	
	while (seci.hasMoreElements())
	{
		secName = seci.peekNextKey();
		ConfigFile::SettingsMultiMap *settings = seci.getNext();
		ConfigFile::SettingsMultiMap::iterator i;
		for (i = settings->begin(); i != settings->end(); ++i)
		{
			typeName = i->first;
			archName = i->second;
			ResourceGroupManager::getSingleton().addResourceLocation(archName, typeName, secName);
		}
	}
}
// ============================================================================
void ClientGameManager::setupRenderSystem()
{
	if (!mRoot->showConfigDialog())
		throw Exception(52, "User canceled the config dialog!", "ClientGameManager::setupRenderSystem()");

}
// ============================================================================
void ClientGameManager::createRenderWindow()
{
	 mRenderWindow = mRoot->initialise( true, "Client - Ogre3DNetworkGame" );
}
// ============================================================================
void ClientGameManager::initializeResourceGroups()
{
	//TextureManager::getSingleton().setDefaultNumMipmaps(5);
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
}
// ============================================================================
void ClientGameManager::setupInputSystem()
{
	mRenderWindow = mRoot->getAutoCreatedWindow();
    InputManager::getSingleton().initialise( mRenderWindow );
	WindowEventUtilities::addWindowEventListener( mRenderWindow, this );//yys

    InputManager::getSingleton().addKeyListener( this, "ClientGameManager" );
    InputManager::getSingleton().addMouseListener( this, "ClientGameManager" );

}
// ============================================================================
void ClientGameManager::setupLighting()
{
	SceneManager *mgr = mRoot->getSceneManager("PlayState SceneManager");
	mgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
}
// ============================================================================
void ClientGameManager::startRenderLoop()
{
	mRoot->startRendering();

}
// ============================================================================
bool ClientGameManager::windowClosing(RenderWindow* rw)
{
	requestShutdown();
	return true;
}
// ============================================================================
void ClientGameManager::processShutdown()
{

	// Clean up all the states
    while( !mStates.empty() )
	{
        mStates.back()->exit();
        mStates.pop_back();
    }

	Ogre::SceneManager * killMe = 
		Root::getSingleton().createSceneManager(ST_EXTERIOR_CLOSE, "temp SceneManager");

	GuiManager::getSingleton().shutdown();

	Root::getSingleton().destroySceneManager(killMe);

}
// ============================================================================
