
#include <Ogre.h>

#include "GameOverState.h"
#include "PlayState.h"
#include "GuiManager.h"
#include "InputManager.h"
#include "OgreConsole.h"
#include "ClientDataManager.h"

using namespace Ogre;

GameOverState * GameOverState::mGameOverState;

// ============================================================================
void GameOverState::update( unsigned long lTimeElapsed )
{
	float timeSinceInSeconds = (float)lTimeElapsed / 1000;

	mCamera->pitch(Degree(timeSinceInSeconds));
	mCamera->yaw(Degree(timeSinceInSeconds));
	
	ClientDataManager::getSingleton().update(timeSinceInSeconds);
}
// ============================================================================
void GameOverState::setupScene()
{
	mSceneMgr = mRoot->createSceneManager(ST_EXTERIOR_CLOSE, "GameOverState SceneManager");
	mCamera = mSceneMgr->createCamera("Camera");
	mCamera->setPosition(Vector3(0,0,100));
    mCamera->lookAt(Vector3(0,0,0));
	mViewport = mRoot->getAutoCreatedWindow()->addViewport(mCamera);

	// Scene:
	mViewport->setBackgroundColour(ColourValue(1.0, 1.0, 1.0));
	mSceneMgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
	mSceneMgr->setSkyBox(true,"Examples/SpaceSkyBox");
}
// ============================================================================
void GameOverState::enter( void )
{
	mIsActive = true;

	srand((unsigned)time(0));
	mRoot = Root::getSingletonPtr();
	this->setupScene();
	
	// Its easier to write 'gui' than 'GuiManager::getSingleton().getGuiPtr()' 
	MyGUI::Gui * gui = GuiManager::getSingleton().getGuiPtr();
	gui->setSceneManager(mSceneMgr);


	// OgreConsole
	//OgreConsole::getSingleton().init(mRoot);
	OgreConsole::getSingleton().setVisible(false);

	widgetPtr = MyGUI::LayoutManager::getInstance().load("ClientGameOverState.layout");
	gui->findWidget<MyGUI::Widget>("Background")->setSize(gui->getViewSize());

	// set callbacks 
	MyGUI::ButtonPtr replay = gui->findWidget<MyGUI::Button>("Replay");
	replay->eventMouseButtonClick = MyGUI::newDelegate(this, &GameOverState::buttonRequestReconnect);

	MyGUI::ButtonPtr exit = gui->findWidget<MyGUI::Button>("Exit");
	exit->eventMouseButtonClick = MyGUI::newDelegate(this, &GameOverState::buttonRequestShutdown);


	GuiManager::getSingleton().getGuiPtr()->showPointer();	

}
// ============================================================================
void GameOverState::exit( void )
{
	MyGUI::LayoutManager::getInstance().unloadLayout(widgetPtr);
	mRoot->getAutoCreatedWindow()->removeViewport(0);

	SceneManager* killMe = mRoot->getSceneManager("GameOverState SceneManager");
	mRoot->destroySceneManager(killMe);

	mIsActive = false;
}
// ============================================================================
void GameOverState::pause( void )
{
}
// ============================================================================
void GameOverState::resume( void )
{
}

// ============================================================================
void GameOverState::keyPressed( const OIS::KeyEvent &e )
{
	// show/hide console window:
	if( e.key == OIS::KC_GRAVE || e.key == OIS::KC_KANJI) // keys: ~ or `
	{
		OgreConsole::getSingleton().setVisible( !OgreConsole::getSingleton().isVisible() );
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyPress( e );
}
// ============================================================================
void GameOverState::keyReleased( const OIS::KeyEvent &e )
{
	if( e.key == OIS::KC_ESCAPE)
	{
		this->requestShutdown();
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyRelease( e );
}
// ============================================================================
void GameOverState::mouseMoved( const OIS::MouseEvent &e )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseMove(e.state.X.abs,e.state.Y.abs, 300);
}
// ============================================================================
void GameOverState::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMousePress(e,id);
}
// ============================================================================
void GameOverState::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseRelease(e,id);
}
// ============================================================================
GameOverState* GameOverState::getSingletonPtr( void )
{
	if( !mGameOverState )
	{
		mGameOverState = new GameOverState();
	}

	return mGameOverState;
}
// ============================================================================
void GameOverState::buttonRequestShutdown(MyGUI::WidgetPtr _widget)
{
	this->requestShutdown();
}
// ============================================================================
void GameOverState::buttonRequestReconnect(MyGUI::WidgetPtr _widget)
{	
	ClientDataManager::getSingleton().init();

	ClientDataManager::getSingleton().reconnect();

	//this->changeState(PlayState::getSingletonPtr());

	// We have to wait until a server accepts our request
	// First wait for a ConnectionAccepted packet, then have to wait for
	// a SendNewPlayerData packet including player data
	// ClientDataManager::handlePacketSendNewPlayerData(...)
}
// ============================================================================

