
#include <Ogre.h>

#include "IntroState.h"
#include "PlayState.h"
#include "GuiManager.h"
#include "InputManager.h"
#include "OgreConsole.h"
#include "ClientDataManager.h"

using namespace Ogre;

IntroState * IntroState::mIntroState;

// ============================================================================
void IntroState::update( unsigned long lTimeElapsed )
{
	float timeSinceInSeconds = (float)lTimeElapsed / 1000;

	mCamera->pitch(Degree(timeSinceInSeconds));
	mCamera->yaw(Degree(timeSinceInSeconds));
	
	ClientDataManager::getSingleton().update(timeSinceInSeconds);
}
// ============================================================================
void IntroState::setupScene()
{
	mSceneMgr = mRoot->createSceneManager(ST_EXTERIOR_CLOSE, "IntroState SceneManager");
	mCamera = mSceneMgr->createCamera("Camera");
	mCamera->setPosition(Vector3(0,0,100));
    mCamera->lookAt(Vector3(0,0,0));
	mViewport = mRoot->getAutoCreatedWindow()->addViewport(mCamera);

	// Scene:
	mViewport->setBackgroundColour(ColourValue(1.0, 1.0, 1.0));
	mSceneMgr->setAmbientLight(ColourValue(1.0f, 1.0f, 1.0f));
	mSceneMgr->setSkyBox(true,"Examples/SpaceSkyBox");
}
// ============================================================================
void IntroState::enter( void )
{
	mIsActive = true;

	srand((unsigned)time(0));
	mRoot = Root::getSingletonPtr();
	this->setupScene();
	
	// Its easier to write 'gui' than 'GuiManager::getSingleton().getGuiPtr()' 
	MyGUI::Gui * gui = GuiManager::getSingleton().getGuiPtr();
	gui->setSceneManager(mSceneMgr);

	//SceneManager* killMe = mRoot->getSceneManager("PlayState SceneManager");
	//mRoot->destroySceneManager(killMe);

	// OgreConsole
	OgreConsole::getSingleton().init(mRoot);
	OgreConsole::getSingleton().setVisible(false);

	widgetPtr = MyGUI::LayoutManager::getInstance().load("ClientIntroState.layout");
	gui->findWidget<MyGUI::Widget>("Background")->setSize(gui->getViewSize());

	// set callbacks 
	MyGUI::ButtonPtr connect = gui->findWidget<MyGUI::Button>("Connect");
	connect->eventMouseButtonClick = MyGUI::newDelegate(this, &IntroState::buttonRequestConnect);

	MyGUI::ButtonPtr exit = gui->findWidget<MyGUI::Button>("Exit");
	exit->eventMouseButtonClick = MyGUI::newDelegate(this, &IntroState::buttonRequestShutdown);


	GuiManager::getSingleton().getGuiPtr()->showPointer();	

}
// ============================================================================
void IntroState::exit( void )
{
	MyGUI::LayoutManager::getInstance().unloadLayout(widgetPtr);
	mRoot->getAutoCreatedWindow()->removeViewport(0);

	SceneManager* killMe = mRoot->getSceneManager("IntroState SceneManager");
	mRoot->destroySceneManager(killMe);

	mIsActive = false;
}
// ============================================================================
void IntroState::pause( void )
{
}
// ============================================================================
void IntroState::resume( void )
{
}

// ============================================================================
void IntroState::keyPressed( const OIS::KeyEvent &e )
{
	// show/hide console window:
	if( e.key == OIS::KC_GRAVE || e.key == OIS::KC_KANJI) // keys: ~ or `
	{
		OgreConsole::getSingleton().setVisible( !OgreConsole::getSingleton().isVisible() );
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyPress( e );
}
// ============================================================================
void IntroState::keyReleased( const OIS::KeyEvent &e )
{
	if( e.key == OIS::KC_ESCAPE)
	{
		this->requestShutdown();
	}

	GuiManager::getSingleton().getGuiPtr()->injectKeyRelease( e );
}
// ============================================================================
void IntroState::mouseMoved( const OIS::MouseEvent &e )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseMove(e.state.X.abs,e.state.Y.abs, 300);
}
// ============================================================================
void IntroState::mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMousePress(e,id);
}
// ============================================================================
void IntroState::mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id )
{
	GuiManager::getSingleton().getGuiPtr()->injectMouseRelease(e,id);
}
// ============================================================================
IntroState* IntroState::getSingletonPtr( void )
{
	if( !mIntroState )
	{
		mIntroState = new IntroState();
	}

	return mIntroState;
}
// ============================================================================
void IntroState::buttonRequestShutdown(MyGUI::WidgetPtr _widget)
{
	this->requestShutdown();
}
// ============================================================================
void IntroState::buttonRequestConnect(MyGUI::WidgetPtr _widget)
{	
	ClientDataManager::getSingleton().init();

	std::string serverIp;
	std::string playerName;

	serverIp = GuiManager::getSingleton().getGuiPtr()->findWidget<MyGUI::Edit>("ServerIpEdit")->getOnlyText();
	playerName = GuiManager::getSingleton().getGuiPtr()->findWidget<MyGUI::Edit>("PlayerNameEdit")->getOnlyText();
	
	ClientDataManager::getSingleton().connect(serverIp,playerName);

	//this->changeState(PlayState::getSingletonPtr());
	// We have to wait until a server accepts our request
	// First wait for a ConnectionAccepted packet, then have to wait for
	// a SendNewPlayerData packet including player data
	// ClientDataManager::handlePacketSendNewPlayerData(...)
}
// ============================================================================

