#ifndef ServerDataManager_H
#define ServerDataManager_H

#include <list>
#include <map>
#include <string>

#include "ServerNetworkNode.h"
#include "Player.h"
#include "Enemy.h"
#include "packetID.h"
#include "Missile.h"

using namespace std;

class ServerDataManager
{
public:
	virtual ~ServerDataManager() {};

	static ServerDataManager & getSingleton();
	void init();
	void processInput(unsigned long timeSinceInMilis);

	// Important method:
	void update(unsigned long timeSinceInMilis);
	
	bool isInitialized() { return initialized; };

	void addPlayer(char colour, string playerName, string entityId);

	string createNewEnemy(); // returns Entity ID of a new, random enemy
	void destroyEnemy(string entityId);

	void handlePacketTest(sockaddr_in * senderAddress, const char * buffer, int bufferMaxSize);
	void handlePacketRequestConnection(sockaddr_in * clientAddress, const char * data, int dataSize);
	void handlePacketRawPlayerData(sockaddr_in * clientAddress, const char * data, int dataSize);
	void handlePacketPlayerRequestsExit(sockaddr_in * clientAddress, const char * data, int dataSize);
	void handlePacketPlayerRequestsCreateMissile(sockaddr_in * clientAddress, const char * data, int dataSize);

	ServerNetworkNode & getNetworkNode() { return mNetworkNode; };

	std::list<Player> mPlayers;

	typedef std::map<string, Enemy> EnemyMapType;
	EnemyMapType mEnemies;
	unsigned int nextEnemyNumber;
	unsigned int numberOfEnemyPacketsToSendEachFrame;

	// Local missile copies (+Dead Reckoning, of course)
	typedef std::map<string, Missile> MissileMapType;
	MissileMapType mMissiles;
	unsigned int nextMissileNumber;
	unsigned int numberOfMissilePacketsToSendEachFrame;



	bool isPacketLossActive;
	bool isDeadReckoningActive;

protected:
	ServerDataManager();
	bool initialized;

	std::list<std::pair<std::string, bool> > mColoursTaken;

	// networking:
	ServerNetworkNode mNetworkNode;

	int lastNumberOfBytesRead;
    char lastReadDataBuf[MAXBUFFERSIZE];


};

#endif