#ifndef GameManager_H
#define GameManager_H

#include <OIS/OIS.h>

#include "InputManager.h"
#include "GameState.h"

class GameManager : public OIS::KeyListener, OIS::MouseListener, public Ogre::WindowEventListener
{
public:
    virtual ~GameManager();

    void startGame( GameState *gameState );

    void changeState( GameState *gameState );
    void pushState( GameState *gameState );
    void popState();
    void requestShutdown();
	void processShutdown();

    static GameManager & getSingleton();

protected:
	GameManager();
    GameManager( const GameManager& ) { }
    GameManager & operator = ( const GameManager& );

    bool configureGame();
    void setupResources();

    bool keyPressed( const OIS::KeyEvent &e );
    bool keyReleased( const OIS::KeyEvent &e );

    bool mouseMoved( const OIS::MouseEvent &e );
    bool mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id );
    bool mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id );

	// from Ogre::WindowEventListener interface class:
	bool windowClosing(Ogre::RenderWindow* rw);

	  

    Ogre::Root				* mRoot;
    Ogre::RenderWindow		* mRenderWindow;
	Ogre::SceneManager		* mgr;
	Ogre::Viewport			* vp;
	Ogre::Camera			* cam;

    bool bShutdown;
    std::vector<GameState*> mStates;

    void createRoot();
    void terrainStuff();
    void defineResources();
    
	void loadManagers();
	void loadParticles();
    void setupRenderSystem();
       
    void createRenderWindow();
   
    void initializeResourceGroups();
   
    void setupScene();
    
    void setupInputSystem();
   
    void setupCEGUI();
   
    void createFrameListener();

	void loadModel();

    void startRenderLoop();

	void setupLighting();

};
#endif