
#ifndef Missile_H
#define Missile_H

#include <Ogre.h>

using namespace Ogre;

class Missile
{
public:
	Missile(void);
	~Missile(void);

	// release resources:
	void destroy(Ogre::String & sceneManagerName);

	const String	mClassName;

	Entity			*mMesh;
	SceneNode		*mSceneNode;

	Ogre::String	getOwner() { return mOwner; };
	void			setOwner(Ogre::String _owner);

	Ogre::String	getId() { return mId; };
	void			setId(Ogre::String _id);

	Ogre::Vector3	getPosition() { return mSceneNode->getPosition(); };
	void			setPosition(Ogre::Vector3 & _position);

	Ogre::Quaternion getOrientation() { return mSceneNode->getOrientation(); };
	void			setOrientation(Ogre::Quaternion & _orientation);

	Ogre::Vector3	getVelocity() { return mVelocity; };
	void			setVelocity(Ogre::Vector3 & _velocity);

	Ogre::Vector3	getAcceleration() { return mAcceleration; }
	void			setAcceleration(Ogre::Vector3 & _acceleration);

	// Accelerate in our local direction
	void			processAccceleration(unsigned long timeSinceInMilis, float multiplier = 1.0f);

	// Change the velocity according to the new acceleration
	void			processVelocity(unsigned long timeSinceInMilis);
	
	// Apply the velocity to change position
	void			processPosition(unsigned long timeSinceInMilis);

	// Create (read) a new missile from a network packet command
	bool initializeFromPacket( const char * networkOrderedBuffer, Ogre::String sceneManager );
	
	// Send the creation (position, ID, etc) through the network
	bool writeInitializationDataToPacket( char * networkOrderedBuffer );

	// Update coords, velocity etc from incoming packet
	void readDataFromPacket(const char * networkOrderedBuffer );

	// Send coords, velocity etc
	void saveDataToPacket( char * networkOrderedBuffer );

	Missile & operator=(Missile & _other);

	bool isInitialized() { return mIsInitialized; };

	const float mMaxSpeed;
	const float mMaxAcceleration;

	unsigned long mCurrentFlyingTimeInMillis;

protected:
	String		mOwner; // player name
	String		mId;

	Vector3		mVelocity;
	Vector3		mAcceleration;

private:
	bool mIsInitialized;
};

#endif
